#include "../KSGKlient.h"
#include "../KSGServer.h"
#include <Datei.h>
#include "../../Global/Variablen.h"
#include <GSLDateiV.h>

typedef GSL::GSLDateiV *( *GetGSLDatei )( );

class DownloadSound : public GSL::GSLSoundV
{
private:
    bool istM;
    int sample;
    __int64 ln;
    Klient *k;
    FBalken *f;
    int ref;

public:
    // Konstruktor
    DownloadSound( Klient *zK, FBalken *zF )
    {
        k = zK;
        f = zF;
        ref = 1;
    }
    // nicht constant
    // GSL
    void playSound() override
    {}
    void setPause( bool p ) override
    {}
    void stopSound() override
    {}
    void warteAufSound( int zeit ) override
    {}
    // Lautstrke: 0 - 0xFFFF
    void setLautstrke( unsigned int links, unsigned int rechts ) override
    {}
    // zum Speichern
    void ffnen() override
    {
        char m = 0;
        k->getNachrichtVerschlsselt( &m, 1 );
        istM = ( m == 1 );
        sample = 0;
        k->getNachrichtVerschlsselt( (char*)&sample, 4 );
        ln = 0;
        k->getNachrichtVerschlsselt( (char*)&ln, 8 );
        f->setAktionAnzahl( ln );
    }
    int getDaten( char *buffer, int ln ) override
    {
        int l = ln < this->ln ? ln : ( int )this->ln;
        if( !l )
            return -1;
        k->getNachricht( buffer, l );
        this->ln -= l;
        f->aktionPlus( l );
        return l;
    }
    void schlieen() override
    {}
    bool istMono() const override
    {
        return istM;
    }
    int getSampleRate() const override
    {
        return sample;
    }
    __int64 getDatLnge() const override
    {
        return ln;
    }
    // Reference Counting
    GSL::GSLSoundV *getThis() override
    {
        ref++;
        return this;
    }
    GSL::GSLSoundV *release() override
    {
        ref--;
        if( !ref )
            delete this;
        return 0;
    }
};

// Inhalt der EditorKlient Klasse aus KSGKlient.h
// Konstruktor
EditorKlient::EditorKlient()
{
    verbunden = 0;
    klient = 0;
    fehler = new Text( "" );
    klientId = getKlientId();
    ref = 1;
    InitializeCriticalSection( &cs );
}

// Destruktor
EditorKlient::~EditorKlient()
{
    LOCK( &cs );
    if( verbunden )
        trenne();
    fehler = fehler->release();
    UNLOCK( &cs );
    DeleteCriticalSection( &cs );
}

// nicht constant
bool EditorKlient::verbinde( unsigned short port, char *ip ) // verbindet ich mit dem Editor Server
{
    if( klient && verbunden && klient->getServerPort() == port && Text( ip ).istGleich( klient->getServerIp() ) )
        return 1;
    if( klient || verbunden )
        trenne();
    klient = new Klient();
    unsigned char schlssel[ 20 ] = { 55, 124, 19, 204, 23, 5, 59, 75, 247, 138, 119, 111, 57, 250, 206, 187, 165, 6, 247, 151 };
    klient->setSendeSchlssel( (char*)schlssel, 20 );
    klient->setEmpfangSchlssel( (char*)schlssel, 20 );
    if( !klient->verbinde( (unsigned short)port, ip ) )
    {
        klient = klient->release();
        fehler->setText( "Fehler beim verbinden mit dem Editor Server." );
        return 0;
    }
    klient->sendeVerschlsselt( "\1", 1 );
    klient->sendeVerschlsselt( (char*)&klientId, 4 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 3 )
    {
        char ln = 0;
        klient->getNachrichtVerschlsselt( &ln, 1 );
        char *nachricht = new char[ ln + 1 ];
        nachricht[ ln ] = 0;
        klient->getNachrichtVerschlsselt( nachricht, ln );
        fehler->setText( nachricht );
        delete[]nachricht;
        trenne();
        return 0;
    }
    if( ret == 1 )
    {
        char *sl = 0;
        char slLn = getSchlssel( &sl );
        klient->setSendeSchlssel( sl, slLn );
        klient->setEmpfangSchlssel( sl, slLn );
        delete[] sl;
        verbunden = 1;
    }
    return 1;
}

bool EditorKlient::karteErstellen( Text *zName, int spielArt ) // Erstellt eine neue Karte
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\xA", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        char l = (char)zName->getLnge();
        klient->sendeVerschlsselt( &l, 1 );
        klient->sendeVerschlsselt( zName->getText(), l );
        klient->sendeVerschlsselt( (char*)&spielArt, 4 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::ladeKarte( int id ) // ldt eine bestimmte Karte
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x6", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( (char*)&id, 4 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 1;
}

int EditorKlient::getAbbildListe( RCArray< Text > *name, RCArray< Zeit > *datum )
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x5", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            int anz = 0;
            klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
            for( int i = 0; i < anz; i++ )
            {
                char l = 0;
                klient->getNachrichtVerschlsselt( &l, 1 );
                char *n = new char[ l + 1 ];
                n[ l ] = 0;
                klient->getNachrichtVerschlsselt( n, l );
                name->set( new Text( n ), i );
                delete[] n;
                Zeit *dat = new Zeit();
                int d = 0;
                klient->getNachrichtVerschlsselt( (char*)&d, 4 );
                dat->setJahr( d );
                klient->getNachrichtVerschlsselt( (char*)&d, 4 );
                dat->setMonat( d );
                klient->getNachrichtVerschlsselt( (char*)&d, 4 );
                dat->setTag( d );
                klient->getNachrichtVerschlsselt( (char*)&d, 4 );
                dat->setStunde( d );
                klient->getNachrichtVerschlsselt( (char*)&d, 4 );
                dat->setMinute( d );
                klient->getNachrichtVerschlsselt( (char*)&d, 4 );
                dat->setSekunde( d );
                datum->set( dat, i );
            }
            UNLOCK( &cs );
            return anz;
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::abbildErstellen( char *name )
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x1", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            char l = (char)textLnge( name );
            klient->sendeVerschlsselt( &l, 1 );
            if( l )
                klient->sendeVerschlsselt( name, l );
            klient->getNachrichtVerschlsselt( &ret, 1 );
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 1;
}

bool EditorKlient::abbildLschen( char *name )
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x2", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            char l = (char)textLnge( name );
            klient->sendeVerschlsselt( &l, 1 );
            if( l )
                klient->sendeVerschlsselt( name, l );
            klient->getNachrichtVerschlsselt( &ret, 1 );
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 1;
}

bool EditorKlient::abbildHerstellen( char *name )
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x3", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            char l = (char)textLnge( name );
            klient->sendeVerschlsselt( &l, 1 );
            if( l )
                klient->sendeVerschlsselt( name, l );
            klient->getNachrichtVerschlsselt( &ret, 1 );
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 1;
}

bool EditorKlient::verffentlichen()
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x4", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
            klient->getNachrichtVerschlsselt( &ret, 1 );
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::shopSeiteVerffentlichen() // Verffentlicht die Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x6", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
            klient->getNachrichtVerschlsselt( &ret, 1 );
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::ladeShopSeiteVorschau() // Lht die Vorschau der Shop Seite herrunter
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x8", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            int anz = 0;
            klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
            Text pf = "data/tmp/ke/ssv";
            for( int i = 0; i < anz; i++ )
            {
                char l = 0;
                klient->getNachrichtVerschlsselt( &l, 1 );
                char *pfad = new char[ l + 1 ];
                pfad[ l ] = 0;
                klient->getNachrichtVerschlsselt( pfad, l );
                Text p = pf.getText();
                p += pfad;
                delete[] pfad;
                Datei d;
                d.setDatei( p );
                d.erstellen();
                d.ffnen( Datei::Style::schreiben );
                __int64 gre = 0;
                klient->getNachrichtVerschlsselt( (char*)&gre, 8 );
                char *buffer = new char[ 2048 ];
                while( gre > 0 )
                {
                    int l = gre > 2048 ? 2048 : (int)gre;
                    klient->getNachricht( buffer, l );
                    d.schreibe( buffer, l );
                    gre -= l;
                }
                delete[] buffer;
                d.schlieen();
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::ssDateiErstellen( char *name, int typ ) // Erstellt eine neue Shop Seiten Datei (typ:0=Ordner,1=Bild,2=Text)
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x1", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char t = typ;
                klient->sendeVerschlsselt( &t, 1 );
                char l = (char)textLnge( name );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( name, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::ssOrdnerffnen( char *name ) // ffnet einen Ordner der Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x2", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)textLnge( name );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( name, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

Bild *EditorKlient::ssBildLaden( char *datei, char *bild ) // Lht das ein Bild der Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x3", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)textLnge( datei );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( datei, l );
                l = (char)textLnge( bild );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( bild, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    Bild *ret = new Bild();
                    int br = 0;
                    int h = 0;
                    klient->getNachrichtVerschlsselt( (char*)&br, 4 );
                    klient->getNachrichtVerschlsselt( (char*)&h, 4 );
                    ret->neuBild( br, h, 0 );
                    char *buffer = (char*)ret->getBuffer();
                    __int64 ln = br * h * 4;
                    for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; l = ln > 2048 ? 2048 : (int)ln )
                    {
                        klient->getNachricht( &( buffer[ i ] ), l );
                        i += l;
                        ln -= l;
                    }
                    UNLOCK( &cs );
                    return ret;
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::ssDateiLschen( char *name ) // Lscht eine Datei der Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x4", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)textLnge( name );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( name, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

Text *EditorKlient::ssTextLaden( char *datei ) // Ld eine KSGS Datei der Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\xA", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)textLnge( datei );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( datei, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int ln = 0;
                    klient->getNachrichtVerschlsselt( (char*)&ln, 4 );
                    char *txt = new char[ ln + 1 ];
                    txt[ ln ] = 0;
                    for( int i = 0, l = ln > 2048 ? 2048 : ln; ln > 0; i += l, ln -= l, l = ln > 2048 ? 2048 : ln )
                        klient->getNachricht( &( txt[ i ] ), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                    if( ret == 1 )
                    {
                        Text *t = new Text( txt );
                        delete[] txt;
                        UNLOCK( &cs );
                        return t;
                    }
                    delete[] txt;
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::ssTextSpeichern( char *datei, Text *zText ) // Spechert eine KSGS Datei der Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x5", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)textLnge( datei );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( datei, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int ln = zText->getLnge();
                    klient->sendeVerschlsselt( (char*)&ln, 4 );
                    for( int i = 0, l = ln > 2048 ? 2048 : ln; ln > 0; i += l, ln -= l, l = ln > 2048 ? 2048 : ln )
                        klient->sende( &( zText->getText()[ i ] ), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::ssBildLschen( char *datei, char *bild ) // Lscht ein Bild der Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x6", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)textLnge( datei );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( datei, l );
                l = (char)textLnge( bild );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( bild, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::ssBildSpeichern( char *datei, char *name, Bild *zBild ) // Speichert ein Bild einer ShopSeite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x7", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)textLnge( datei );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( datei, l );
                l = (char)textLnge( name );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( name, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int br = zBild->getBreite();
                    int h = zBild->getHhe();
                    klient->sendeVerschlsselt( (char*)&br, 4 );
                    klient->sendeVerschlsselt( (char*)&h, 4 );
                    __int64 ln = br * h * 4;
                    char *buffer = (char*)zBild->getBuffer();
                    for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; i += l, ln -= l, l = ln > 2048 ? 2048 : (int)ln )
                        klient->sende( &( buffer[ i ] ), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

int EditorKlient::ssGetDateiListe( RCArray< Text > *zList ) // Ld die Datei Liste der Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x8", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                int anz = 0;
                klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
                for( int i = 0; i < anz; i++ )
                {
                    char l = 0;
                    klient->getNachrichtVerschlsselt( &l, 1 );
                    char *n = new char[ l + 1 ];
                    n[ l ] = 0;
                    klient->getNachrichtVerschlsselt( n, l );
                    zList->add( new Text( n ) );
                    delete[] n;
                }
                UNLOCK( &cs );
                return anz;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return -1;
    }
    UNLOCK( &cs );
    return 0;
}

int EditorKlient::ssGetBildListe( char *name, RCArray< Text > *zList ) // Ld die Bild Liste einer Bild Datei der Shop Seite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x7", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x9", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)textLnge( name );
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( name, l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int anz = 0;
                    klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
                    for( int i = 0; i < anz; i++ )
                    {
                        l = 0;
                        klient->getNachrichtVerschlsselt( &l, 1 );
                        char *n = new char[ l + 1 ];
                        n[ l ] = 0;
                        klient->getNachrichtVerschlsselt( n, l );
                        zList->add( new Text( n ) );
                        delete[] n;
                    }
                    UNLOCK( &cs );
                    return anz;
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return -1;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::getShopDaten( int &es, int &tp, int &vp ) // Lht die Shop Daten
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\x9", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->getNachrichtVerschlsselt( (char*)&es, 4 );
            klient->getNachrichtVerschlsselt( (char*)&tp, 4 );
            klient->getNachrichtVerschlsselt( (char*)&vp, 4 );
            UNLOCK( &cs );
            return 1;
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::setShopDaten( int es, int tp, int vp ) // Setzt die Shop Daten
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xA", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( (char*)&es, 4 );
            klient->sendeVerschlsselt( (char*)&tp, 4 );
            klient->sendeVerschlsselt( (char*)&vp, 4 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                UNLOCK( &cs );
                return 1;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

Text *EditorKlient::beschreibungLaden() // lht die Kartenbeschreibung
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xB", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x1", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                int ln = 0;
                klient->getNachrichtVerschlsselt( (char*)&ln, 4 );
                char *txt = new char[ ln + 1 ];
                txt[ ln ] = 0;
                for( int i = 0, l = ln > 2048 ? 2048 : ln; ln > 0; i += l, ln -= l, l = ln > 2048 ? 2048 : ln )
                    klient->getNachricht( &( txt[ i ] ), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    Text *t = new Text( txt );
                    delete[] txt;
                    UNLOCK( &cs );
                    return t;
                }
                delete[] txt;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

Bild *EditorKlient::titelbildLaden() // lht das Titelbild
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xB", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x2", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                Bild *ret = new Bild();
                int br = 0;
                int h = 0;
                klient->getNachrichtVerschlsselt( (char*)&br, 4 );
                klient->getNachrichtVerschlsselt( (char*)&h, 4 );
                ret->neuBild( br, h, 0 );
                char *buffer = (char*)ret->getBuffer();
                __int64 ln = br * h * 4;
                for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; l = ln > 2048 ? 2048 : (int)ln )
                {
                    klient->getNachricht( &( buffer[ i ] ), l );
                    i += l;
                    ln -= l;
                }
                UNLOCK( &cs );
                return ret;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

Bild *EditorKlient::minimapLaden() // lht das Minimapbild
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xB", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x3", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                Bild *ret = new Bild();
                int br = 0;
                int h = 0;
                klient->getNachrichtVerschlsselt( (char*)&br, 4 );
                klient->getNachrichtVerschlsselt( (char*)&h, 4 );
                ret->neuBild( br, h, 0 );
                char *buffer = (char*)ret->getBuffer();
                __int64 ln = br * h * 4;
                for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; l = ln > 2048 ? 2048 : (int)ln )
                {
                    klient->getNachricht( &( buffer[ i ] ), l );
                    i += l;
                    ln -= l;
                }
                UNLOCK( &cs );
                return ret;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

Bild *EditorKlient::ladebildLaden() // lht das Ladebild
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xB", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x4", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                Bild *ret = new Bild();
                int br = 0;
                int h = 0;
                klient->getNachrichtVerschlsselt( (char*)&br, 4 );
                klient->getNachrichtVerschlsselt( (char*)&h, 4 );
                ret->neuBild( br, h, 0 );
                char *buffer = (char*)ret->getBuffer();
                __int64 ln = br * h * 4;
                for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; l = ln > 2048 ? 2048 : (int)ln )
                {
                    klient->getNachricht( &( buffer[ i ] ), l );
                    i += l;
                    ln -= l;
                }
                UNLOCK( &cs );
                return ret;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::beschreibungSpeichern( Text *zText ) // speichert die Kartenbeschreibung
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xB", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x5", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                int ln = zText->getLnge();
                klient->sendeVerschlsselt( (char*)&ln, 4 );
                for( int i = 0, l = ln > 2048 ? 2048 : ln; ln > 0; i += l, ln -= l, l = ln > 2048 ? 2048 : ln )
                    klient->sende( &( zText->getText()[ i ] ), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::titelbildSpeichern( Bild *zBild ) // speichert das Titelbild
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xB", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x6", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                int br = zBild->getBreite();
                int h = zBild->getHhe();
                klient->sendeVerschlsselt( (char*)&br, 4 );
                klient->sendeVerschlsselt( (char*)&h, 4 );
                __int64 ln = br * h * 4;
                char *buffer = (char*)zBild->getBuffer();
                for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; i += l, ln -= l, l = ln > 2048 ? 2048 : (int)ln )
                    klient->sende( &( buffer[ i ] ), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::minimapSpeichern( Bild *zBild ) // speichert das Minimapbild
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xB", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x7", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                int br = zBild->getBreite();
                int h = zBild->getHhe();
                klient->sendeVerschlsselt( (char*)&br, 4 );
                klient->sendeVerschlsselt( (char*)&h, 4 );
                __int64 ln = br * h * 4;
                char *buffer = (char*)zBild->getBuffer();
                for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; i += l, ln -= l, l = ln > 2048 ? 2048 : (int)ln )
                    klient->sende( &( buffer[ i ] ), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::ladebildSpeichern( Bild *zBild ) // speichert das Ladebild
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xB", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x8", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                int br = zBild->getBreite();
                int h = zBild->getHhe();
                klient->sendeVerschlsselt( (char*)&br, 4 );
                klient->sendeVerschlsselt( (char*)&h, 4 );
                __int64 ln = br * h * 4;
                char *buffer = (char*)zBild->getBuffer();
                for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; i += l, ln -= l, l = ln > 2048 ? 2048 : (int)ln )
                    klient->sende( &( buffer[ i ] ), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::ladeTeamDaten( SpielerTeamStruktur *sts ) // lht die Team Daten
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xC", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x1", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                klient->getNachrichtVerschlsselt( (char*)&sts->spielerAnzahl, 4 );
                klient->getNachrichtVerschlsselt( (char*)&sts->teamAnzahl, 4 );
                for( int i = 0; i < sts->spielerAnzahl; i++ )
                {
                    int f = 0;
                    klient->getNachrichtVerschlsselt( (char*)&f, 4 );
                    sts->spielerFarbe->set( f, i );
                }
                for( int i = 0; i < sts->teamAnzahl; i++ )
                {
                    int f = 0;
                    klient->getNachrichtVerschlsselt( (char*)&f, 4 );
                    sts->teamFarbe->set( f, i );
                }
                for( int i = 0; i < sts->teamAnzahl; i++ )
                {
                    char l = 0;
                    klient->getNachrichtVerschlsselt( &l, 1 );
                    char *n = new char[ l + 1 ];
                    n[ l ] = 0;
                    klient->getNachrichtVerschlsselt( n, l );
                    sts->teamName->set( new Text( n ), i );
                    delete[] n;
                }
                for( int i = 0; i < sts->teamAnzahl; i++ )
                {
                    int g = 0;
                    klient->getNachrichtVerschlsselt( (char*)&g, 4 );
                    sts->teamGre->set( g, i );
                }
                UNLOCK( &cs );
                return 1;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::speicherTeamDaten( SpielerTeamStruktur *sts ) // speichert die Team Daten
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xC", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x2", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                klient->sendeVerschlsselt( (char*)&sts->spielerAnzahl, 4 );
                klient->sendeVerschlsselt( (char*)&sts->teamAnzahl, 4 );
                for( int i = 0; i < sts->spielerAnzahl; i++ )
                {
                    int f = sts->spielerFarbe->get( i );
                    klient->sendeVerschlsselt( (char*)&f, 4 );
                }
                for( int i = 0; i < sts->teamAnzahl; i++ )
                {
                    int f = sts->teamFarbe->get( i );
                    klient->sendeVerschlsselt( (char*)&f, 4 );
                }
                for( int i = 0; i < sts->teamAnzahl; i++ )
                {
                    char l = (char)sts->teamName->z( i )->getLnge();
                    klient->sendeVerschlsselt( &l, 1 );
                    klient->sendeVerschlsselt( sts->teamName->z( i )->getText(), l );
                }
                for( int i = 0; i < sts->teamAnzahl; i++ )
                {
                    int g = sts->teamGre->get( i );
                    klient->sendeVerschlsselt( (char*)&g, 4 );
                }
                UNLOCK( &cs );
                return 1;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::deNeueDatei( char typ, Text *zName ) // DateienEditor: Erstellt neue Datei (typ: 0=ordner 1=bild 2=modell2d 3=sound
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x1", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                klient->sendeVerschlsselt( &typ, 1 );
                char l = (char)zName->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( zName->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::deOrdnerffnen( Text *zName ) // DateienEditor: ffnet Ordner
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x2", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char ln = (char)zName->getLnge();
                klient->sendeVerschlsselt( &ln, 1 );
                if( ln )
                    klient->sendeVerschlsselt( zName->getText(), ln );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

Bild *EditorKlient::deBildLaden( Text *zDatei, Text *zBild, FBalken *zF ) // DateienEditor: Bild laden
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    Bild *retB = 0;
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x3", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char ln = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &ln, 1 );
                if( ln )
                    klient->sendeVerschlsselt( zDatei->getText(), ln );
                ln = (char)zBild->getLnge();
                klient->sendeVerschlsselt( &ln, 1 );
                if( ln )
                    klient->sendeVerschlsselt( zBild->getText(), ln );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int br = 0;
                    int h = 0;
                    klient->getNachrichtVerschlsselt( (char*)&br, 4 );
                    klient->getNachrichtVerschlsselt( (char*)&h, 4 );
                    retB = new Bild();
                    retB->neuBild( br, h, 0 );
                    char *buffer = (char*)retB->getBuffer();
                    __int64 ln = br * h * 4;
                    zF->setAktionAnzahl( ln );
                    for( int i = 0, l = ln > 2048 ? 2048 : (int)ln; ln > 0; l = ln > 2048 ? 2048 : (int)ln )
                    {
                        klient->getNachricht( &( buffer[ i ] ), l );
                        i += l;
                        ln -= l;
                        zF->aktionPlus( l );
                    }
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    if( !ret && retB )
        retB = retB->release();
    return retB;
}

bool EditorKlient::deDateiLschen( Text *zName ) // DateienEditor: Lscht Datei
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x4", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char ln = (char)zName->getLnge();
                klient->sendeVerschlsselt( &ln, 1 );
                if( ln )
                    klient->sendeVerschlsselt( zName->getText(), ln );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::deBildLschen( Text *zDatei, Text *zBild ) // DateienEditor: Lscht Bild aus Datei
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x5", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( zDatei->getText(), l );
                l = (char)zBild->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                if( l )
                    klient->sendeVerschlsselt( zBild->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::deBildSpeichern( Text *zDatei, Text *zName, Bild *zBild, FBalken *zF ) // DateienEditor: Speichert Bild
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x6", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char ln = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &ln, 1 );
                if( ln )
                    klient->sendeVerschlsselt( zDatei->getText(), ln );
                ln = (char)zName->getLnge();
                klient->sendeVerschlsselt( &ln, 1 );
                if( ln )
                    klient->sendeVerschlsselt( zName->getText(), ln );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int br = zBild->getBreite();
                    int h = zBild->getHhe();
                    klient->sendeVerschlsselt( (char*)&br, 4 );
                    klient->sendeVerschlsselt( (char*)&h, 4 );
                    char *buffer = (char*)zBild->getBuffer();
                    __int64 gr = br * h * 4;
                    zF->setAktionAnzahl( gr );
                    for( int i = 0, l = gr > 2048 ? 2048 : (int)gr; gr > 0; i += l, gr -= l, l = gr > 2048 ? 2048 : (int)gr )
                    {
                        klient->sende( &( buffer[ i ] ), l );
                        zF->aktionPlus( l );
                    }
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

int EditorKlient::deGetDateiListe( RCArray< Text > *zNamen ) // DateienEditor: Datei Liste herunterladen
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x7", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                int anz = 0;
                klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
                for( int i = 0; i < anz; i++ )
                {
                    char l = 0;
                    klient->getNachrichtVerschlsselt( &l, 1 );
                    char *n = new char[ l + 1 ];
                    n[ l ] = 0;
                    if( l )
                        klient->getNachrichtVerschlsselt( n, l );
                    zNamen->add( new Text( n ) );
                    delete[] n;
                }
                UNLOCK( &cs );
                return anz;
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return -1;
    }
    UNLOCK( &cs );
    return ret == 1;
}

int EditorKlient::deGetBildListe( Text *zDatei, RCArray< Text > *zNamen ) // DateienEditor: Liste mit in der Datei gespeicherten Bildern
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x8", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int anz = 0;
                    klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
                    for( int i = 0; i < anz; i++ )
                    {
                        klient->getNachrichtVerschlsselt( &l, 1 );
                        char *n = new char[ l + 1 ];
                        n[ l ] = 0;
                        if( l )
                            klient->getNachrichtVerschlsselt( n, l );
                        zNamen->add( new Text( n ) );
                        delete[] n;
                    }
                    UNLOCK( &cs );
                    return anz;
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

int EditorKlient::deGetModelListe( Text *zDatei, RCArray< Text > *zNamen ) // DateienEditor Liste mit in der Datei gespeicherten Modellen
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x9", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int anz = 0;
                    klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
                    for( int i = 0; i < anz; i++ )
                    {
                        klient->getNachrichtVerschlsselt( &l, 1 );
                        char *n = new char[ l + 1 ];
                        n[ l ] = 0;
                        if( l )
                            klient->getNachrichtVerschlsselt( n, l );
                        zNamen->add( new Text( n ) );
                        delete[] n;
                    }
                    UNLOCK( &cs );
                    return anz;
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

int EditorKlient::deGetSoundListe( Text *zDatei, RCArray< Text > *zNamen ) // DateienEditor: Liste mit in der Datei gespeicherten Tnen
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\xA", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    int anz = 0;
                    klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
                    for( int i = 0; i < anz; i++ )
                    {
                        klient->getNachrichtVerschlsselt( &l, 1 );
                        char *n = new char[ l + 1 ];
                        n[ l ] = 0;
                        if( l )
                            klient->getNachrichtVerschlsselt( n, l );
                        zNamen->add( new Text( n ) );
                        delete[] n;
                    }
                    UNLOCK( &cs );
                    return anz;
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::deModelLschen( Text *zDatei, Text *zModel ) // DateienEditor: Lscht Modell
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\xB", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    l = (char)zModel->getLnge();
                    klient->sendeVerschlsselt( &l, 1 );
                    klient->sendeVerschlsselt( zModel->getText(), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::deSoundLschen( Text *zDatei, Text *zSound ) // DateienEditor: Lscht Ton
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\xC", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    l = (char)zSound->getLnge();
                    klient->sendeVerschlsselt( &l, 1 );
                    klient->sendeVerschlsselt( zSound->getText(), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

Model2DData *EditorKlient::deModelLaden( Text *zDatei, Text *zModel, FBalken *zF ) // DateienEditor: ld ein Model herunter
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\xD", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    l = (char)zModel->getLnge();
                    klient->sendeVerschlsselt( &l, 1 );
                    klient->sendeVerschlsselt( zModel->getText(), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                    if( ret == 1 )
                    {
                        int anz = 0;
                        klient->getNachrichtVerschlsselt( (char*)&anz, 4 );
                        zF->setAktionAnzahl( anz );
                        Array< Polygon2D > *pol = new Array< Polygon2D >();
                        for( int i = 0; i < anz; i++ )
                        {
                            Polygon2D p;
                            p.vertex = new Array< Vertex >();
                            p.tKordinaten = new Array< Punkt >();
                            int anz2 = 0;
                            klient->getNachrichtVerschlsselt( (char*)&anz2, 4 );
                            for( int j = 0; j < anz2; j++ )
                            {
                                Vertex v;
                                klient->getNachrichtVerschlsselt( (char*)&v.x, 4 );
                                klient->getNachrichtVerschlsselt( (char*)&v.y, 4 );
                                Punkt t;
                                klient->getNachrichtVerschlsselt( (char*)&t.x, 4 );
                                klient->getNachrichtVerschlsselt( (char*)&t.y, 4 );
                                p.vertex->add( v );
                                p.tKordinaten->add( t );
                            }
                            pol->add( p );
                            zF->aktionPlus();
                        }
                        Model2DData *data = new Model2DData();
                        data->erstelleModell( pol );
                        UNLOCK( &cs );
                        return data;
                    }
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

GSL::GSLSoundV *EditorKlient::deSoundLaden( Text *zDatei, Text *zSound, FBalken *zF ) // DateienEditor: ld Sound herunter
{
    HMODULE gslDll = dllDateien->ladeDLL( "GSL.dll", "data/bin/GSL.dll" );
    if( !gslDll )
    {
        fehler->setText( "Die DLL Datei 'data/bin/GSL.dll' wurde nicht gefunden." );
        return 0;
    }
    GetGSLDatei getGSLDatei = (GetGSLDatei)GetProcAddress( gslDll, "getGSLDatei" );
    if( !getGSLDatei )
    {
        fehler->setText( "Der Einstiegspunkt 'getGSLDatei' wurde in der Datei 'data/bin/GSL.dll' nicht gefunden." );
        return 0;
    }
    DateiLschen( "data/tmp/editor/dateien/sounds/tmp.gsl" );
    DateiPfadErstellen( "data/tmp/editor/dateien/sounds/tmp.gsl" );
    GSL::GSLDateiV *gslDatei = getGSLDatei();
    gslDatei->setDatei( "data/tmp/editor/dateien/sounds/tmp.gsl" );
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        gslDatei->release();
        dllDateien->releaseDLL( "GSL.dll" );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\xE", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    l = (char)zSound->getLnge();
                    klient->sendeVerschlsselt( &l, 1 );
                    klient->sendeVerschlsselt( zSound->getText(), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                    if( ret == 1 )
                    {
                        DownloadSound dws( klient, zF );
                        gslDatei->speicherSound( &dws, zSound->getText() );
                        GSL::GSLSoundV *s = gslDatei->getSound( zSound->getText() );
                        gslDatei->release();
                        dllDateien->releaseDLL( "GSL.dll" );
                        UNLOCK( &cs );
                        return s;
                    }
                }
            }
        }
    }
    gslDatei->release();
    dllDateien->releaseDLL( "GSL.dll" );
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return 0;
}

bool EditorKlient::deModelSpeichern( Text *zDatei, Text *zModel, Model2DData *zData, FBalken *zF ) // DateienEditor: Speichert Modell
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\xF", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    l = (char)zModel->getLnge();
                    klient->sendeVerschlsselt( &l, 1 );
                    klient->sendeVerschlsselt( zModel->getText(), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                    if( ret == 1 )
                    {
                        int anz = zData->polygons ? zData->polygons->getEintragAnzahl() : 0;
                        zF->setAktionAnzahl( anz );
                        klient->sendeVerschlsselt( (char*)&anz, 4 );
                        for( int i = 0; i < anz; i++ )
                        {
                            Polygon2D pol = zData->polygons->get( i );
                            int anz2 = pol.vertex->getEintragAnzahl();
                            klient->sendeVerschlsselt( (char*)&anz2, 4 );
                            for( int j = 0; j < anz2; j++ )
                            {
                                Vertex v = pol.vertex->get( j );
                                Punkt p;
                                if( pol.tKordinaten )
                                    p = pol.tKordinaten->hat( j ) ? pol.tKordinaten->get( j ) : Punkt( 0, 0 );
                                klient->sendeVerschlsselt( (char*)&v.x, 4 );
                                klient->sendeVerschlsselt( (char*)&v.y, 4 );
                                klient->sendeVerschlsselt( (char*)&p.x, 4 );
                                klient->sendeVerschlsselt( (char*)&p.y, 4 );
                            }
                            zF->aktionPlus();
                        }
                        klient->getNachrichtVerschlsselt( &ret, 1 );
                    }
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::deSoundSpeichern( Text *zDatei, Text *zSound, GSL::GSLSoundV *zData, FBalken *zF ) // DateienEditor: Speichert Sound
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xD", 1 );
        klient->getNachrichtVerschlsselt( &ret, 1 );
        if( ret == 1 )
        {
            klient->sendeVerschlsselt( "\x10", 1 );
            klient->getNachrichtVerschlsselt( &ret, 1 );
            if( ret == 1 )
            {
                char l = (char)zDatei->getLnge();
                klient->sendeVerschlsselt( &l, 1 );
                klient->sendeVerschlsselt( zDatei->getText(), l );
                klient->getNachrichtVerschlsselt( &ret, 1 );
                if( ret == 1 )
                {
                    l = (char)zSound->getLnge();
                    klient->sendeVerschlsselt( &l, 1 );
                    klient->sendeVerschlsselt( zSound->getText(), l );
                    klient->getNachrichtVerschlsselt( &ret, 1 );
                    if( ret == 1 )
                    {
                        char channels = zData->istMono() ? 1 : 2;
                        klient->sendeVerschlsselt( &channels, 1 );
                        int sample = zData->getSampleRate();
                        klient->sendeVerschlsselt( (char*)&sample, 4 );
                        __int64 sln = zData->getDatLnge();
                        klient->sendeVerschlsselt( (char*)&sln, 8 );
                        zF->setAktionAnzahl( sln );
                        zData->ffnen();
                        char *buffer = new char[ 2048 ];
                        while( sln > 0 )
                        {
                            int l = sln > 2048 ? 2048 : (int)sln;
                            zData->getDaten( buffer, l );
                            klient->sende( buffer, l );
                            sln -= l;
                            zF->aktionPlus( l );
                        }
                        delete[] buffer;
                        zData->schlieen();
                    }
                }
            }
        }
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::initEditor() // Initialisiert den Editor auf Serverseite
{
    LOCK( &cs );
    if( !verbunden )
    {
        UNLOCK( &cs );
        return 0;
    }
    klient->sendeVerschlsselt( "\x7", 1 );
    char ret = 0;
    klient->getNachrichtVerschlsselt( &ret, 1 );
    if( ret == 1 )
    {
        klient->sendeVerschlsselt( "\xE", 1 );
        char ret = 0;
        klient->getNachrichtVerschlsselt( &ret, 1 );
    }
    if( ret == 3 )
    {
        char byte = 0;
        klient->getNachrichtVerschlsselt( &byte, 1 );
        char *f = new char[ byte + 1 ];
        f[ byte ] = 0;
        klient->getNachrichtVerschlsselt( f, byte );
        fehler->setText( f );
        delete[] f;
        UNLOCK( &cs );
        return 0;
    }
    UNLOCK( &cs );
    return ret == 1;
}

bool EditorKlient::keepAlive() // Erhlt die Verbindung aufrecht
{
    if( !TryEnterCriticalSection( &cs ) )
        return 1;
    if( !verbunden || !klient )
    {
        UNLOCK( &cs );
        return 0;
    }
    char res = 0;
    klient->sendeVerschlsselt( "\x5", 1 );
    klient->getNachrichtVerschlsselt( &res, 1 );
    UNLOCK( &cs );
    if( res != 1 )
    {
        trenne();
        fehler->setText( "Verbindung unterbrochen: 'Keep Alive' nicht erfolgreich." );
    }
    return res == 1;
}

bool EditorKlient::trenne() // trennt sich von dem Editor Server
{
    LOCK( &cs );
    if( !klient || !verbunden )
    {
        if( klient )
            klient = klient->release();
        UNLOCK( &cs );
        return 1;
    }
    char serverReturn;
    klient->sendeVerschlsselt( "\4", 1 );
    klient->getNachrichtVerschlsselt( &serverReturn, 1 );
    if( serverReturn == 3 )
    {
        char ln = 0;
        klient->getNachrichtVerschlsselt( &ln, 1 );
        char *nachricht = new char[ ln + 1 ];
        nachricht[ ln ] = 0;
        klient->getNachrichtVerschlsselt( nachricht, ln );
        delete[]nachricht;
    }
    klient->sendeVerschlsselt( "\3", 1 );
    klient->getNachrichtVerschlsselt( &serverReturn, 1 );
    klient->trenne();
    klient = klient->release();
    verbunden = 0;
    UNLOCK( &cs );
    return 1;
}

void EditorKlient::lock()
{
    LOCK( &cs );
}

void EditorKlient::unlock()
{
    UNLOCK( &cs );
}

Klient *EditorKlient::zKlient() // gibt den Klient zurck
{
    if( cs.LockCount && cs.OwningThread == GetCurrentThread() )
        return klient;
    return 0;
}

// constant
bool EditorKlient::istVerbunden() const // prft, ob mit Editor Server verbunden
{
    return verbunden;
}

char *EditorKlient::getLetzterFehler() const // gibt den Letzten Fehlertext zuck
{
    return fehler->getText();
}

// Reference Counting
EditorKlientV *EditorKlient::getThis()
{
    ref++;
    return this;
}

EditorKlientV *EditorKlient::release()
{
    ref--;
    if( !ref )
        delete this;
    return 0;
}